@@ -3,7 +3,8 @@ from django.contrib import admin
 from django.contrib.admin import SimpleListFilter
 from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext_lazy as _
-from avocado.models import DataField, DataConcept, DataCategory, DataConceptField
+from avocado.models import (DataField, DataConcept, DataCategory,
+    DataConceptField, DataView, DataContext, DataQuery)
 from avocado.forms import DataFieldAdminForm
 
 
@@ -106,12 +107,16 @@ class DataFieldAdmin(PublishedAdmin):
     list_display = ('name', 'published', 'archived', 'internal',
         'orphan_status', 'model_name', 'enumerable',
         'is_lexicon', 'is_objectset', 'related_dataconcepts')
+
     list_filter = ('published', 'archived', 'internal', 'model_name',
         'enumerable', LexiconListFilter, ObjectSetListFilter)
+
     list_editable = ('published', 'archived', 'internal', 'enumerable')
 
     search_fields = ('name', 'description', 'keywords')
-    readonly_fields = ('created', 'modified', 'data_modified')
+
+    readonly_fields = ('created', 'modified', 'data_version')
+
     actions = ('mark_published', 'mark_unpublished', 'mark_archived',
         'mark_unarchived', 'create_dataconcept_multi',
         'create_dataconcept_single')
@@ -149,7 +154,7 @@ class DataFieldAdmin(PublishedAdmin):
         }),
 
         ('Times of Interest', {
-            'fields': ('created', 'modified', 'data_modified'),
+            'fields': ('created', 'modified'),
         }),
     )
 
@@ -206,10 +211,15 @@ class DataConceptFieldInlineAdmin(admin.TabularInline):
 
 class DataConceptAdmin(PublishedAdmin):
     list_display = ('name', 'published', 'archived', 'internal', 'category',
-        'queryview', 'formatter_name', 'sortable', 'related_datafields')
-    list_editable = ('published', 'archived', 'internal', 'category',
-        'queryview', 'formatter_name', 'sortable')
-    list_filter = ('published', 'archived', 'category')
+        'order', 'formatter_name', 'viewable', 'queryable', 'sortable',
+        'related_datafields')
+
+    list_editable = ('published', 'archived', 'internal', 'category', 'order',
+        'formatter_name', 'viewable', 'queryable', 'sortable')
+
+    list_filter = ('published', 'archived', 'category', 'formatter_name',
+        'viewable', 'queryable', 'sortable')
+
     inlines = [DataConceptFieldInlineAdmin]
 
     fieldsets = (
@@ -220,6 +230,7 @@ class DataConceptAdmin(PublishedAdmin):
                 'description',
                 'keywords',
                 'category',
+                'order',
                 'published',
                 'archived',
             ),
@@ -232,7 +243,7 @@ class DataConceptAdmin(PublishedAdmin):
         }),
 
         ('Modifiers', {
-            'fields': ('queryview', 'formatter_name', 'sortable'),
+            'fields': ('formatter_name', 'viewable', 'queryable', 'sortable'),
         }),
 
         ('Times of Interest', {
@@ -254,7 +265,151 @@ class DataCategoryAdmin(admin.ModelAdmin):
 
     list_display = ('name', 'parent', 'order')
 
+    list_editable = ('parent', 'order')
+
+
+class DataViewAdmin(admin.ModelAdmin):
+    readonly_fields = ('session_key', 'sql')
+
+    fieldsets = (
+        (None, {
+            'fields': (
+                'name',
+                'description',
+                'keywords',
+            ),
+        }),
+
+        ('User', {
+            'fields': ('user', 'session_key', 'session')
+        }),
+
+        ('Template', {
+            'fields': ('template', 'default')
+        }),
+
+        ('JSON', {
+            'fields': ('json',),
+            'description': 'JSON representation of the view',
+        }),
+
+        ('SQL', {
+            'fields': ('sql',),
+            'description': 'Formatted SQL representation of the view',
+        }),
+    )
+
+    def sql(self, obj):
+        sql = obj.sql()
+        try:
+            import sqlparse
+            sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
+        except ImportError:
+            pass
+        return u'<pre>{0}</pre>'.format(sql)
+    sql.short_description = 'SQL'
+    sql.allow_tags = True
+
+
+class DataContextAdmin(admin.ModelAdmin):
+    readonly_fields = ('session_key', 'count', 'sql')
+
+    fieldsets = (
+        (None, {
+            'fields': (
+                'name',
+                'description',
+                'keywords',
+            ),
+        }),
+
+        ('User', {
+            'fields': ('user', 'session_key', 'session')
+        }),
+
+        ('Template', {
+            'fields': ('template', 'default')
+        }),
+
+        ('Stats', {
+            'fields': ('count',)
+        }),
+
+        ('JSON', {
+            'fields': ('json',),
+            'description': 'JSON representation of the context',
+        }),
+
+        ('SQL', {
+            'fields': ('sql',),
+            'description': 'Formatted SQL representation of the context',
+        }),
+    )
+
+    def sql(self, obj):
+        sql = obj.sql()
+        try:
+            import sqlparse
+            sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
+        except ImportError:
+            pass
+        return u'<pre>{0}</pre>'.format(sql)
+    sql.short_description = 'SQL'
+    sql.allow_tags = True
+
+
+
+class DataQueryAdmin(admin.ModelAdmin):
+    readonly_fields = ('distinct_count', 'record_count', 'session_key', 'sql')
+
+    fieldsets = (
+        (None, {
+            'fields': (
+                'name',
+                'description',
+                'keywords',
+            ),
+        }),
+
+        ('User', {
+            'fields': ('user', 'session_key', 'session')
+        }),
+
+        ('Template', {
+            'fields': ('template', 'default')
+        }),
+
+        ('Stats', {
+            'fields': ('distinct_count', 'record_count')
+        }),
+
+        ('JSON', {
+            'fields': ('view_json', 'context_json'),
+            'description': 'JSON representation of the context and view',
+        }),
+
+        ('SQL', {
+            'fields': ('sql',),
+            'description': 'Formatted SQL representation of the query',
+        }),
+    )
+
+    def sql(self, obj):
+        sql = obj.sql()
+        try:
+            import sqlparse
+            sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
+        except ImportError:
+            pass
+        return u'<pre>{0}</pre>'.format(sql)
+    sql.short_description = 'SQL'
+    sql.allow_tags = True
+
 
 admin.site.register(DataField, DataFieldAdmin)
 admin.site.register(DataConcept, DataConceptAdmin)
 admin.site.register(DataCategory, DataCategoryAdmin)
+
+admin.site.register(DataView, DataViewAdmin)
+admin.site.register(DataContext, DataContextAdmin)
+admin.site.register(DataQuery, DataQueryAdmin)
