@@ -3,7 +3,7 @@
 # only need to differentiate between high-level types like number, string,
 # and boolean. More granular separation be may desired to alter the
 # allowed operators or may infer a different client-side representation
-SIMPLE_TYPE_MAP = {
+SIMPLE_TYPES = {
     'auto': 'key',
     'foreignkey': 'key',
 
@@ -32,14 +32,14 @@ SIMPLE_TYPE_MAP = {
 # that will be used to validate a query condition. In many cases, these types
 # support more operators than what are defined, but are not include because
 # they are not commonly used.
-OPERATOR_MAP = {
+OPERATORS = {
     'key': ('exact', '-exact', 'in', '-in'),
     'boolean': ('exact', '-exact', 'in', '-in'),
-    'date': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range'),
-    'number': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range'),
+    'date': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range', '-range'),
+    'number': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range', '-range'),
     'string': ('exact', '-exact', 'iexact', '-iexact', 'in', '-in', 'icontains', '-icontains'),
-    'datetime': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range'),
-    'time': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range'),
+    'datetime': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range', '-range'),
+    'time': ('exact', '-exact', 'in', '-in', 'lt', 'lte', 'gt', 'gte', 'range', '-range'),
 }
 
 # A general mapping of formfield overrides for all subclasses. the mapping is
@@ -61,7 +61,7 @@ INTERNAL_DATATYPE_FORMFIELDS = {
 ENUMERABLE_MAXIMUM = 30
 
 # Flag for enabling the history API
-HISTORY_ENABLED = False
+HISTORY_ENABLED = True
 
 # The maximum size of a user's history. If the value is an integer, this
 # is the maximum number of allowed items in the user's history. Set to
@@ -90,3 +90,11 @@ QUERY_PROCESSORS = {
 # Toggles for enabling search
 CONCEPT_SEARCH_ENABLED = True
 FIELD_SEARCH_ENABLED = False
+
+# Custom validation error and warnings messages
+VALIDATION_ERRORS = {}
+VALIDATION_WARNINGS = {}
+
+# Toggle whether DataField instances should cache the underlying data
+# for their most common data access methods.
+DATA_CACHE_ENABLED = True
