@@ -32,8 +32,13 @@ class Migration(DataMigration):
 
 log = logging.getLogger(__name__)
 
+
+_help = """\
+Simple utility for dumping Avocado metadata.
+"""
+
 class Command(BaseCommand):
-    help = 'Simple utility for dumping Avocado metadata.'
+    __doc__ = help = _help
 
     option_list = BaseCommand.option_list + (
         make_option('--database', action='store', dest='database',
@@ -57,7 +62,7 @@ class Command(BaseCommand):
             raise CommandError(e.message)
 
         if not os.path.exists(fixture_dir):
-            os.makedir(fixture_dir)
+            os.makedirs(fixture_dir)
             log.info(u'Created fixture directory: {0}'.format(fixture_dir))
         elif not os.path.isdir(fixture_dir):
             raise CommandError(u'The metadata fixture directory {0}.. is not a directory.'.format(fixture_dir))
