@@ -4,12 +4,12 @@ from collections import defaultdict
 
 def std_dev(values):
     """
-    Computes the standard deviation of the 'values' list. 
+    Computes the standard deviation of the 'values' list.
 
     TODO: Consider adding support for axes as in numpy and letting this method
     accept nested lists:
         http://docs.scipy.org/doc/numpy/reference/generated/numpy.std.html
-    
+
     Arguments:
         values: list
             A list of numbers to compute the standard deviation of.
@@ -33,7 +33,7 @@ def is_iterable(obj):
     must, by definition, define one of these attributes but is not required or
     guarenteed to define both of them. For more information on iterable types
     see:
-        
+
         http://docs.python.org/2/glossary.html#term-iterable
         http://docs.python.org/2/library/functions.html#iter
     """
@@ -70,7 +70,7 @@ def divide_lists(lst_numer, lst_denom):
         lst_denom: list of denominators(dimensions 1 x M)
 
     Returns:
-        A new list formed by dividing each element of 'lst_numer' by 
+        A new list formed by dividing each element of 'lst_numer' by
         'lst_denom' according to the division process described above.
     """
     indexes = range(len(lst_denom))
@@ -84,7 +84,7 @@ def normalize(points):
 
     NOTE: If all the points are the same along any dimension, the resulting
     normalized value of all points along that dimension will be 0 because the
-    standard deviation of a collection of equal values is 0 and we can't 
+    standard deviation of a collection of equal values is 0 and we can't
     realistically divide by 0 so we instead set the normalized value to 0.
 
     Arguments:
@@ -96,7 +96,7 @@ def normalize(points):
             >>> points = [[ 1., 2., 3.],  #  p0
             ...           [ 4., 5., 6.],  #  p1
             ...           [ 7., 8., 9.]]  #  p2
-            
+
             Single dimension point list:
             #             d0
             >>> points = [1.,   #p0
@@ -112,9 +112,9 @@ def normalize(points):
         The values in 'points' scaled by the standard deviation along each
         dimension.
     """
-    # Check for a single dimension list. This check assumes that if the first 
-    # element is not a list then all elements are non-list and the list is 
-    # single dimension. If the list is single dimension just divide all the 
+    # Check for a single dimension list. This check assumes that if the first
+    # element is not a list then all elements are non-list and the list is
+    # single dimension. If the list is single dimension just divide all the
     # points by the standard deviation and return that as the normalized list.
     if len(points) > 0 and not is_iterable(points[0]):
         std = std_dev(points)
@@ -123,14 +123,14 @@ def normalize(points):
         else:
             return divide_by_scalar(points, std_dev(points))
 
-    # Organize the points list as a list where each row is a list of values of 
+    # Organize the points list as a list where each row is a list of values of
     # the same dimension.
     dimensions = zip(*points)
 
     # Compute the standard deviation of each dimension.
     std = [std_dev(d) for d in dimensions]
-    
-    # Divide all the point dimensions by the standard deviation of that 
+
+    # Divide all the point dimensions by the standard deviation of that
     # dimension over all points.
     if 0 in std:
         norm_points = []
@@ -143,10 +143,10 @@ def normalize(points):
                     norm_point.append(p / s)
 
             norm_points.append(norm_point)
-        
+
         return norm_points
     else:
-        return divide_lists(points, std) 
+        return divide_lists(points, std)
 
 def is_nested(points):
     """
@@ -192,15 +192,15 @@ def get_dimension(points):
         >>> get_dimension(p)
         1
 
-    This method checks that all points have the same dimension and will raise 
+    This method checks that all points have the same dimension and will raise
     a ValueError if all the points do not share the same dimension.
 
     Arguments:
         points: list of points of any dimension
 
     Returns:
-        The dimension of the point list(aka the number of columns) or raises a 
-        ValueError if the number of dimensions is not consistent over all 
+        The dimension of the point list(aka the number of columns) or raises a
+        ValueError if the number of dimensions is not consistent over all
         points in 'points'.
     """
 
@@ -215,12 +215,12 @@ def get_dimension(points):
             i_dimension = len(points[i])
         else:
             i_dimension = 1
-        
+
         if i_dimension != dimension:
             raise ValueError("Points must have the same number of dimensions.")
 
     return dimension
-    
+
 def sqr_euclidean_dist(point1, point2):
     """
     Calculates the square Euclidean distance between all dimension of 2 points.
@@ -233,14 +233,14 @@ def sqr_euclidean_dist(point1, point2):
         >>> p2 = [3, 6, 9]
         >>> sqr_euclidean_dist(p1, p2)
         [4, 14, 36]
-    
+
         >>> o1 = 5.4
         >>> o2 = 2.4
         >>> sqr_euclidean_dist(p1, p2)
         9.000000000000004
 
-    NOTE: This method assumes 'point1' and 'point2' are both numbers or lists 
-    of equal length. No verification is done to make sure that both have the 
+    NOTE: This method assumes 'point1' and 'point2' are both numbers or lists
+    of equal length. No verification is done to make sure that both have the
     same type or that the list lengths are equal.
 
     Arguments:
@@ -263,12 +263,12 @@ def index_of_min(values):
     """
     min_value = float('Inf')
     min_index = 0
-    
+
     for i in range(len(values)):
         if values[i] < min_value:
             min_value = values[i]
             min_index = i
-    
+
     return min_index
 
 def compute_clusters(points, centroids):
@@ -279,7 +279,7 @@ def compute_clusters(points, centroids):
     the training set. This method can be considered the encoder part of the
     vector quantization algorithm where the codebook that is generated contains
     the cluster(centroid index) that each point belongs to and the distance
-    between each point and its cluster centroid. The distance referred to 
+    between each point and its cluster centroid. The distance referred to
     above is the Euclidean distance.
 
     For some general info on vector quantization theory see:
@@ -287,19 +287,19 @@ def compute_clusters(points, centroids):
         http://www.data-compression.com/vq.html
 
     Returns:
-        clusters: A list such that clusters[i] is the index of the centroid 
+        clusters: A list such that clusters[i] is the index of the centroid
                   of the cluster that the ith point is a member of. This list
                   will have the same length as 'points'.
-        distances: A list such that distances[i] represents the Euclidean 
+        distances: A list such that distances[i] represents the Euclidean
                    distance between the ith point and the centroid of the
-                   cluster it is a member of. This list will have the same 
+                   cluster it is a member of. This list will have the same
                    length as 'points'.
     """
     if len(points) < 1:
         raise ValueError("cluster requires at least one point.")
     if len(centroids) < 1:
         raise ValueError("cluster requires at least one centroid.")
-    
+
     d = get_dimension(points)
     n = len(points)
     cluster_indexes = range(len(centroids))
@@ -310,26 +310,25 @@ def compute_clusters(points, centroids):
     clusters = [0] * n
     min_distances = [0] * n
     for i in range(n):
-        # Compute the squared Euclidean distance between this point and each 
+        # Compute the squared Euclidean distance between this point and each
         # of the centroids.
         distances = [sqr_euclidean_dist(points[i], centroids[j]) \
                 for j in cluster_indexes]
-        
+
         # Sum across all dimensions of each distance measure. If we only have
         # one dimension then this is simply the distance list itself.
         if d == 1 and not is_nested(centroids):
             distance_totals = distances
         else:
-            distance_totals = \
-                    [sum(distances[j]) for j in range(len(distances))]
+            distance_totals = [sum(distances[j]) for j in range(len(distances))]
 
-        # Find the index containing the minimum distance. 
+        # Find the index containing the minimum distance.
         clusters[i] = index_of_min(distance_totals)
 
         min_distances[i] = distance_totals[clusters[i]]
-        
+
     # Do the square root now to get the Euclidean distance. We don't do this
-    # in the loop so that we can save time by not taking the square root of 
+    # in the loop so that we can save time by not taking the square root of
     # non-minimal distances.
     return clusters, [math.sqrt(d) for d in min_distances]
 
@@ -337,8 +336,8 @@ def dimension_mean(points):
     """
     Calculates the mean of the points along each dimension.
 
-    If the points are of a single dimension, 'points' will just be a list of 
-    numbers and the mean of the list itself will be returned. For more 
+    If the points are of a single dimension, 'points' will just be a list of
+    numbers and the mean of the list itself will be returned. For more
     details, see the example below:
 
         >>> p = [[1.0, 0.0],
@@ -356,7 +355,7 @@ def dimension_mean(points):
     # Organize the points list as a list where each row is a list of values
     # of the same dimension.
     dimensions = zip(*points)
-    
+
     return [sum(d) / float(len(d)) for d in dimensions]
 
 def kmeans(points, k_or_centroids, threshold=1e-5):
@@ -365,8 +364,8 @@ def kmeans(points, k_or_centroids, threshold=1e-5):
 
     This method runs the k-means clustering algorithm assigning the 'points'
     to k clusters. k is determined in one of two ways depending on what is
-    provided in 'k_or_centroids'. If 'k_or_centroids' is a list, then the 
-    contents are used as the centroids and k is the length of that list. If 
+    provided in 'k_or_centroids'. If 'k_or_centroids' is a list, then the
+    contents are used as the centroids and k is the length of that list. If
     'k_or_centroids' is not a list of centroids it is assumed to be an integer
     to be used as k and the initial centroids are then k random points chosen
     from 'points'.
@@ -383,7 +382,7 @@ def kmeans(points, k_or_centroids, threshold=1e-5):
     # if initial centroids are provided.
     if len(points) < 1:
         raise ValueError("points must contain at least 1 point.")
-   
+
     if is_iterable(k_or_centroids):
         k = len(k_or_centroids)
         initial_centroids = k_or_centroids
@@ -400,8 +399,8 @@ def kmeans(points, k_or_centroids, threshold=1e-5):
     while mean_difference > threshold:
         num_centroids = len(centroids)
 
-        # Determine cluster membership and distances for all points given the 
-        # current centroids. 
+        # Determine cluster membership and distances for all points given the
+        # current centroids.
         centroid_indexes, distances = compute_clusters(points, centroids)
 
         # Compute the mean distance of all points to their corresponding
@@ -414,18 +413,18 @@ def kmeans(points, k_or_centroids, threshold=1e-5):
             mean_difference = previous_mean_distance - mean_distance
 
         # The following is the update step of the k-means algorithm where the
-        # centroid position changes based on the points in each cluster. We 
-        # can safely ignore this step if the have reached the threshold for 
+        # centroid position changes based on the points in each cluster. We
+        # can safely ignore this step if the have reached the threshold for
         # minimum difference in mean distance.
         if mean_difference > threshold:
             for i in range(num_centroids):
                 # Get all the points that are currently members of this
-                # centroid's cluster. 
+                # centroid's cluster.
                 members = [p for c, p in \
                         zip(centroid_indexes, points) if c == i]
 
-                # If the cluster has any points in it then update the centroid 
-                # of that cluster to be a point represented by the mean of all 
+                # If the cluster has any points in it then update the centroid
+                # of that cluster to be a point represented by the mean of all
                 # points in that cluster along each dimension.
                 if len(members) > 0:
                     centroids[i] = dimension_mean(members)
@@ -436,33 +435,33 @@ def kmeans(points, k_or_centroids, threshold=1e-5):
             centroids = [centroids[i] for i in \
                     range(len(centroids)) if centroids[i]]
 
-        # Store this mean distance so we can access it in the next loop and 
+        # Store this mean distance so we can access it in the next loop and
         # diff against this iterations mean distance.
         previous_mean_distance = mean_distance
-    
+
     return centroids, previous_mean_distance
 
 def find_outliers(points, outlier_threshold=3, normalized=True):
     """
     Finds and returns outliers in the 'points'.
 
-    Outliers are those items in the 'points' that have a normalized distance 
-    that is at least 'outlier_threshold' away from the the calculated center 
+    Outliers are those items in the 'points' that have a normalized distance
+    that is at least 'outlier_threshold' away from the the calculated center
     of the population defined in 'points'. The normalized distance of a
-    point is the distance from the point to its cluster's centroid divided by 
+    point is the distance from the point to its cluster's centroid divided by
     the mean distance of all point-to-centroid distances.
 
     Arguments:
-        points: list of n-dimensional points 
+        points: list of n-dimensional points
             An NxM list of lists defining the population to check for outliers
-            where M is the number of dimensions per point and N is the number 
+            where M is the number of dimensions per point and N is the number
             of points.
         outlier_threshold: float
             Used to define outliers. Outliers are points with normalized
             distances greater than this threshold.
         normalized: bool
             Value indicating whether 'points' have already been normalized or
-            not. Passing False for this argument will result in the original 
+            not. Passing False for this argument will result in the original
             'points' being normalized before the outlier search starts.
 
     Returns:
@@ -473,58 +472,57 @@ def find_outliers(points, outlier_threshold=3, normalized=True):
 
     d = get_dimension(points)
 
-    # Organize the points list as a list where each row is a list of values of 
+    # Organize the points list as a list where each row is a list of values of
     # the same dimension.
     if is_nested(points):
         dimensions = zip(*points)
 
-    # Compute the mid-point index and construct the centroid by taking the 
+    # Compute the mid-point index and construct the centroid by taking the
     # midpoint of the sorted list in each dimension.
     midpoint_index = (len(points) - 1) / 2
-    
+
     if d == 1 and not is_nested(points):
         centroids = [sorted(points)[midpoint_index]]
     else:
         centroids = [[sorted(d)[midpoint_index] for d in dimensions]]
 
     # Calculate the distance from every point to the centroid.
-    _, distances = \
-            compute_clusters(points, kmeans(points, centroids)[0])
-    
+    _, distances = compute_clusters(points, kmeans(points, centroids)[0])
+
     mean_distance = sum(distances) / float(len(distances))
-    
+
     return [i for i, distance in enumerate(distances) \
-            if (distance / mean_distance) >= outlier_threshold]
+            if mean_distance > 0 and (distance / mean_distance) >= outlier_threshold]
 
 def kmeans_optm(points, k=None, outlier_threshold=3):
     """
-    Execute k-means clustering(for finding centroid) on, compute the clusters 
+    Execute k-means clustering(for finding centroid) on, compute the clusters
     for, and run outlier detection on the 'points' population.
 
     Returns a dictionary containing the following data:
         'centroids': The k centers of the k clusters found by running the
                      k-means algorithm.
-        'indexes':   An ordered list mapping each point in 'points' to the 
+        'indexes':   An ordered list mapping each point in 'points' to the
                      centroid of the cluster it falls in.
-        'distances': A list of distances between each point and the centroid 
+        'distances': A list of distances between each point and the centroid
                      of its cluster.
-        'outliers':  A list of outlier indexes in the 'points' list. This is 
+        'outliers':  A list of outlier indexes in the 'points' list. This is
                      only returned if an 'outlier_threshold' is specified. If
                      outliers aren't required, set outlier_threshold=None.
 
     If 'k' is not supplied it will be calculated given the following formula:
-        
+
         k = sqrt(n / 2)
 
-    where n is the length of the 'points' list. The initialization method is 
-    optimized to find the mid-points of each cluster from a sorted list of 
-    points. This ultimately reduces the number of iterations required during 
+    where n is the length of the 'points' list. The initialization method is
+    optimized to find the mid-points of each cluster from a sorted list of
+    points. This ultimately reduces the number of iterations required during
     clustering due to increased likelihood of convergence.
 
     Referenes:
         Number of clusters - http://en.wikipedia.org/wiki/Determining_the_number_of_clusters_in_a_data_set#Rule_of_thumb
         Improved k-means initial centroids - http://www.ijcsit.com/docs/vol1issue2/ijcsit2010010214.pdf
-        
+
     See:
         find_outliers()
 
@@ -537,74 +535,73 @@ def kmeans_optm(points, k=None, outlier_threshold=3):
             Used to define outliers. Outliers are points with normalized
             distances greater than this threshold.
     Returns:
-        A dictionary containing the centroids, indexes of each point's 
-        centroid, distances from each point to its centroid and optionally a 
-        list of indexes of the outliers in 'points'. More detailed info on 
+        A dictionary containing the centroids, indexes of each point's
+        centroid, distances from each point to its centroid and optionally a
+        list of indexes of the outliers in 'points'. More detailed info on
         the return values are included above.
     """
-    # If an outlier threshold is defined, remove outliers relative to the 
+    # If an outlier threshold is defined, remove outliers relative to the
     # population before running k-means clustering.
     if outlier_threshold:
         outliers = find_outliers(points, outlier_threshold, False)
-        points = \
-                [p for i, p in enumerate(points) if i not in outliers]
+        points = [p for i, p in enumerate(points) if i not in outliers]
     else:
         outliers = []
 
     d = get_dimension(points)
     n = len(points)
-    
+
     # Compute the number of clusters if it wasn't specified.
     k = k or int(math.sqrt(n / 2))
 
-    # Manually calculate the standard deviation for each dimension of the 
-    # point list in order to de-normalize the centroids later. Otherwise, the 
-    # centroid would be relative to the normalized dimensions rather than the 
+    # Manually calculate the standard deviation for each dimension of the
+    # point list in order to de-normalize the centroids later. Otherwise, the
+    # centroid would be relative to the normalized dimensions rather than the
     # original.
     if d == 1 and not is_nested(points):
         std = std_dev(points)
     else:
-        # Organize the points list as a list where each row is a list of 
+        # Organize the points list as a list where each row is a list of
         # valuesof the same dimension.
         dimensions = zip(*points)
         std = [std_dev(d) for d in dimensions]
- 
+
     norm_points = normalize(points)
 
     if d == 1 and not is_nested(points):
         sorted_points = sorted(norm_points)
     else:
-        # Organize the normalized list as a list where each row is a list of 
+        # Organize the normalized list as a list where each row is a list of
         # values of the same dimension.
         norm_dimensions = zip(*norm_points)
 
-        # Sort the points by dimension and then return the dimension based 
-        # list back to the original point form so the sorted list has n 
+        # Sort the points by dimension and then return the dimension based
+        # list back to the original point form so the sorted list has n
         # records and m dimensions just like the original points list.
         sorted_dimensions = [sorted(d) for d in norm_dimensions]
         sorted_points = [list(d) for d in zip(*sorted_dimensions)]
-    
+
     step = n / k
     offset = step / 2
     initial_centroids = sorted_points[offset::step]
 
-    # Perform the clustering then calculate the centroid each point is 
-    # associated with and the distance between each point and its centroid. 
-    # Centroids returned here are based on the normalized(see normalize()) 
+    # Perform the clustering then calculate the centroid each point is
+    # associated with and the distance between each point and its centroid.
+    # Centroids returned here are based on the normalized(see normalize())
     # points.
     centroids, _ = kmeans(norm_points, initial_centroids)
     indexes, distances = compute_clusters(norm_points, centroids)
 
-    # Denormalize the centroids for downstream use. Do this by multiplying 
-    # each dimension of each centroid by the standard deviation along that 
+    # Denormalize the centroids for downstream use. Do this by multiplying
+    # each dimension of each centroid by the standard deviation along that
     # dimension that we calculated earlier.
     if d == 1 and not is_nested(points):
         denorm_centroids = [c * std for c in centroids]
     else:
         denorm_centroids = [[d * s for d, s in zip(c, std)] for c in centroids]
-    
+
     return {
-        'centroids': denorm_centroids, 
+        'centroids': denorm_centroids,
         'indexes': indexes,
         'distances': distances,
         'outliers': outliers,
@@ -616,27 +613,27 @@ def weighted_counts(points, counts, k):
 
     Performs k-means clustering in order to identify the centroids and the
     distances between all points and their centroids. Weights and distances
-    for the centroids are then constructed from all the points in that 
+    for the centroids are then constructed from all the points in that
     centroids cluster. Using those values, the weighted count is computed for
     each of the centroids and returned along with the centroid themselves. The
-    points returned are a list of point objects where each point in the list 
+    points returned are a list of point objects where each point in the list
     has a list of 'values' representing the centroid coordinates and a 'count'
     which is the weighted count as computed for that centroid. Also returned
-    is the list of outlier indexes in the 'points' list as determined by the 
+    is the list of outlier indexes in the 'points' list as determined by the
     k-means algorithm.
 
     Arguments:
         points - list of n-dimensional points to run k-means on.
-        counts - the count for each point in the 'points' list. This list 
+        counts - the count for each point in the 'points' list. This list
                  must be of the same length as the 'points' list because the
                  count for a point is determined by accessing both 'points'
                  and 'counts' with the same index.
         k - the number of clusters(k-value) to send to the k-means algorithm.
 
     Returns:
-        centroid_counts - A list of points where each point has 'values' list 
-        containing the coordinates of the centroid it is associated with and a 
-        'count' which is an integer representing the weighted count as 
+        centroid_counts - A list of points where each point has 'values' list
+        containing the coordinates of the centroid it is associated with and a
+        'count' which is an integer representing the weighted count as
         computed for that centroid.
         outliers - A list of indexes of the outliers in 'point's as returned
         by the k-means algorithm.
@@ -671,5 +668,5 @@ def weighted_counts(points, counts, k):
             'values': values,
             'count': int(sum(weighted_counts)),
         })
-    
+
     return centroid_counts, outliers
